package docar.archive.document;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * 문서 파일을 분석하는 클래스.
 * 
 * @author "Sunny Kwak"
 */
public class ContentAnalyzer {
	private static final String HTTP_PROTOCOL = "http";

	private URL location;
	private String fileName;
	private long fileSize;
	private String checksum;

	public ContentAnalyzer(URL location) {
		this.location = location;
	}

	public void analyze() {
		InputStream inputStream = null;

		try {
			String protocol = location.getProtocol();
			if (protocol == null
					|| !protocol.toLowerCase().equals(HTTP_PROTOCOL)) {
				inputStream = new FileInputStream(new java.io.File(location.toURI()));
			} else {
				inputStream = location.openConnection().getInputStream();
			}

			fileName = location.getFile();
			fileSize = 0L;
			checksum = getMD5Checksum(inputStream);

		} catch (IOException e) {
			// FIXME add logging
			e.printStackTrace();
		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException ignored) {
					// do nothing
				}
			}
		}

	}

	public String getFileName() {
		return fileName;
	}

	public String getChecksum() {
		return checksum;
	}

	public long getSize() {
		return fileSize;
	}

	public String getAbstraction() {
		// FIXME 요약생성 구현 필요함.
		return null;
	}

	public Format getFormat() {
		// FIXME 문서 형식 체크 기능 구현 필요.
		return Format.HTML;
	}

	private byte[] createChecksum(InputStream inputStream) throws IOException {
		byte[] buffer = new byte[1024];
		MessageDigest complete = null;
		try {
			complete = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException ignored) {
			// do nothing..
		}
		int numRead;

		do {
			numRead = inputStream.read(buffer);
			if (numRead > 0) {
				complete.update(buffer, 0, numRead);
				fileSize += numRead;
			}
		} while (numRead != -1);

		return complete.digest();
	}

	private String getMD5Checksum(InputStream inputStream) throws IOException {
		byte[] b = createChecksum(inputStream);
		String result = "";
		for (int i = 0; i < b.length; i++) {
			result += Integer.toString((b[i] & 0xff) + 0x100, 16).substring(1);
		}
		return result;
	}
}
