package docar.cli;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import docar.archive.document.Document;
import docar.archive.document.DocumentBuilder;
import docar.storage.volume.DirectoryFilter;
import docar.storage.volume.DiskVolume;
import docar.storage.volume.DocumentFilter;
import docar.storage.volume.EmptyFileFilter;
import docar.user.Writer;
import docar.util.profile.Profiler;
import docar.util.profile.Report;
import docar.util.profile.StopWatch;

public class DuplicationFinder {

	private static final String TARGET_VOLUME = "D:/";
	
	public static void main(String[] args) {
		
		StopWatch stopWatch = new StopWatch("DuplicationFinder.main");

		Writer guessWho = new Writer("GuessWho");

		// MS-Office, PDF, 텍스트 등의 문서만 검색
		DocumentFilter documentFilter = new DocumentFilter();

		// 휴지통 폴더 제외
		DirectoryFilter directoryFilter = new DirectoryFilter();
		directoryFilter.addNotAllowed("$RECYCLE.BIN");
		
		// 크기가 0인 파일 제외
		EmptyFileFilter emptyFileFilter = new EmptyFileFilter();

		DiskVolume diskVolume = new DiskVolume(new File(TARGET_VOLUME));
		diskVolume.addFilter(documentFilter);
		diskVolume.addFilter(directoryFilter);
		diskVolume.addFilter(emptyFileFilter);
		diskVolume.open();

		// 볼륨의 전체 파일 목록 <체크섬, 문서>
		Map<String, Document> documentsMap = new HashMap<String, Document>();
		// 중복 파일 목록 <체크섬, 문서목록>
		Map<String, List<Document>> duplicatedSetMap = new HashMap<String, List<Document>>();

		long count = 0L;
		URL url;
		while ((url = diskVolume.nextFileURL()) != null) {
			DocumentBuilder documentBuilder = new DocumentBuilder();
			documentBuilder.setLocation(url);
			documentBuilder.setWriter(guessWho);

			Document document = documentBuilder.createDocument();
			docar.archive.document.File docFile = document.getFiles().get(0);
			String checksum = docFile.getChecksum();

			// 이미 동일한 체크섬을 가진 파일이 존재할 경우...
			if (documentsMap.containsKey(checksum)) {
				// 중복 목록에 이미 존재하면 추가
				if (duplicatedSetMap.containsKey(checksum)) {
					duplicatedSetMap.get(checksum).add(document);
				}
				// 아니면, 새롭게 중복 목록 작성.
				else {
					List<Document> duplicatedSet = new ArrayList<Document>();
					duplicatedSet.add(documentsMap.get(checksum));
					duplicatedSet.add(document);
					duplicatedSetMap.put(checksum, duplicatedSet);
				}
			}
			// 새로운 파일인 경우, 전체 문서 목록에 추가.
			else {
				documentsMap.put(checksum, document);
			}

			// 10건 스캔할 때마다, hash tag 출력 (진행 상태 표시)
			if (++count % 10 == 0) {
				System.out.print("#");
			}
			// 50개의 hash 태그를 출력할 때마다, 줄바꿈... (진행 상태 표시)
			if (count % 500 == 0) {
				System.out.println();
			}
		}
		
		Profiler.getInstance().addCallEvent(stopWatch.stop());
		Report.report(System.out);
		
		System.out.println();

		// 중복 파일 목록 출력
		for (List<Document> documentSet : duplicatedSetMap.values()) {
			for (Document doc : documentSet) {
				docar.archive.document.File file = doc.getFiles().get(0);
				try {
					System.out.println(new File(file.getLocation().toURI())
							.getAbsolutePath());
				} catch (URISyntaxException ignored) {
					// do nothing
				}

			}
			System.out.println();
		}

	}

}
