package docar.storage.volume;

import java.net.URL;

/**
 * <pre>
 * 원본 파일 저장 볼륨 인터페이스.
 * 
 * 디스크 혹은 웹사이트를 추상화한 인터페이스이며, 해당 볼륨에 포함된 파일 목록을 반환하는 기능을 제공한다.
 * </pre>
 * 
 * @author "Sunny Kwak"
 *
 */
public interface Volume {

	/**
	 * 저장소의 식별자(ID)를 반환
	 * 
	 * @return 저장소 식별자
	 */
	URL getURL();
	
	/**
	 * 스캔 필터 설정
	 * 
	 * @param filter 스캔 필터
	 */
	void addFilter(Filter filter);

	/**
	 * 볼륨의 파일들을 스캔(scan)하기 위해 준비.
	 * 
	 * @param filter 파일 필터
	 */
	void open();

	/**
	 * 볼륨의 다음 파일 반환.
	 * 
	 * @return 파일 URL. 더 이상 스캔할 파일이 없을 경우, null 반환.
	 */
	URL nextFileURL();

	/**
	 * 볼륨 스캔 종료
	 */
	void close();
}
